#include "mylib.h"

#include <xlw/XlfServices.h>
#include <xlw/XlfExcel.h>

#include <math.h>
#include <time.h>

double MySqrt(double x)
{
  if (x < 0.)
  {
    throw("#Negative argument");
  }
  return sqrt(x);
}

double SumSqrt(double x, double y)
{
  if (x < 0 || y < 0)
  {
    throw("#Negative argument");
  }
  return sqrt(x) + sqrt(y);
}

MyMatrix Outer1(const MyArray& a)
{
  MyMatrix res(a.size(), a.size());
  for (size_t i = 0; i < a.size(); ++i)
  {
    for (size_t j = 0; j < a.size(); ++j)
    {
      res[i][j] = a[i] * a[j];
    }
  }

  return res;
}

XlfOper SquareNumbers(XlfOper& input)
{
  if( !input.IsMulti() )
  {
    XlfOper res;
    if (input.IsNumber())
    {
      const double x = input.AsDouble();
      res = x * x;
    }
    else
    {
      res = "-";
    }
    return res;
  }

  XlfOper res(input.rows(), input.columns());
  for (size_t i = 0; i < res.rows(); ++i)
  {
    for (size_t j = 0; j < res.columns(); ++j)
    {
      if(input(i,j).IsNumber())
      {
        const double x = input(i, j).AsDouble();
        res(i, j) = x*x;
      }
      else
      {
        res(i, j) = "-";
      }
    }
  }
  return res;
}

MyMatrix CallingRange(double k)
{
  const size_t nrows = XlfServices.Information.GetCallingCell().rows();
  const size_t ncols = XlfServices.Information.GetCallingCell().columns();

  MyMatrix res(nrows, ncols);

  for (size_t i = 0; i < nrows; ++i )
  {
    for (size_t j = 0; j < ncols; ++j )
    {
      res[i][j] = pow((nrows - i)*(ncols - j), k);
    }
  }

  return res;
}

double EscInterrupt(int n)
{
  double x = 3;
  for (int i = 0; i < n; ++i)
  {
    x = sqrt(x);
    x *= x;
    if( XlfExcel::Instance().IsEscPressed() )
    {
      return double(i);
    }
  }
  return double(n);
}

double SysTime()
{
  time_t timer = time(NULL);

  struct tm y2k;
  y2k.tm_hour = 0;   y2k.tm_min = 0; y2k.tm_sec = 0;
  y2k.tm_year = 100; y2k.tm_mon = 0; y2k.tm_mday = 1;

  double seconds = difftime(timer, mktime(&y2k));

  return seconds/60;
}
